#include <stdlib.h>
#include <stdio.h>
#include "SDL/SDL.h"
#include "SDL/SDL_video.h"

#include "rgba32.h"
#include "sdlstuff.h"

#define PIXEL_DEPTH 32

void sdls_init(unsigned int width, unsigned int height) {
    if (SDL_Init(SDL_INIT_VIDEO) < 0) {
        fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
        exit(1);
    }
    atexit(sdls_cleanup);

    SDL_Surface * window_surface = SDL_SetVideoMode(width, height, PIXEL_DEPTH, SDL_SWSURFACE | SDL_DOUBLEBUF);
    if (window_surface == NULL) {
        fprintf(stderr, "Couldn't create window: %s\n", SDL_GetError());
        exit(1);
    }
}

void sdls_cleanup(void) {
    SDL_Quit();
}


static void sdls_setpixel(SDL_Surface * surface, unsigned int x, unsigned int y, rgba32 color) {
    Uint32 value = SDL_MapRGBA(surface->format, color.r, color.g, color.b, color.a);
    Uint32 * pixel = (Uint32*) surface->pixels + y * (surface->pitch / surface->format->BytesPerPixel) + x;
    *pixel = value;
}

void sdls_drawrectangle(unsigned int x, unsigned int y, unsigned int width, unsigned int height, rgba32 * src) {
    SDL_Surface * window_surface = SDL_GetVideoSurface();

    if (SDL_MUSTLOCK(window_surface)) {
        SDL_LockSurface(window_surface);
    }

    rgba32 * cursor = src;
    for (unsigned int row = y; row < y + height; ++row) {
        for (unsigned int col = x; col < x + width; ++col) {
            sdls_setpixel(window_surface, col, row, *cursor);
            ++cursor;
        }
    }

    if (SDL_MUSTLOCK(window_surface)) {
        SDL_UnlockSurface(window_surface);
    }

    SDL_Flip(window_surface);
}
